package com.lz.modules.sys.service.app.impl;

import com.baomidou.mybatisplus.extension.service.impl.ServiceImpl;
import com.lz.modules.sys.dao.app.ResultDetailMapper;
import com.lz.modules.sys.entity.app.ResultDetail;
import com.lz.modules.sys.service.app.ResultDetailService;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;

import java.util.List;

/**
* <p>
* 业绩详情表 服务类
* </p>
*
* @author quyixiao
* @since 2020-08-10
*/

@Service
public class ResultDetailServiceImpl extends ServiceImpl<ResultDetailMapper, ResultDetail> implements ResultDetailService {


    @Autowired
	private ResultDetailMapper resultDetailMapper;



	@Override
	public ResultDetail selectResultDetailById(Long id){
		return resultDetailMapper.selectResultDetailById(id);
	}



	@Override
	public Long insertResultDetail(ResultDetail resultDetail){
		return resultDetailMapper.insertResultDetail(resultDetail);
	}



	@Override
	public int updateResultDetailById(ResultDetail resultDetail){
		return resultDetailMapper.updateResultDetailById(resultDetail);
	}



	@Override
	public int updateCoverResultDetailById(ResultDetail resultDetail){
		return resultDetailMapper.updateCoverResultDetailById(resultDetail);
	}



	@Override
	public int deleteResultDetailById(Long id){
		return resultDetailMapper.deleteResultDetailById(id);
	}


    @Override
    public List<ResultDetail> selectByRecordId(Long recordResultId) {
        return resultDetailMapper.selectByRecordId(recordResultId);
    }

	@Override
	public List<ResultDetail> selectByRecordIdType(Long recordResultId, Integer type) {
		return resultDetailMapper.selectByRecordIdType(recordResultId,type);
	}


}
